//*******************************************************
//
//	LiPoLight
//
//	ATtiny13
//		hfuse:0xFB	BOD 2.7V
//		lfuse:22h	1.2MHz
//
//	Compiler:	AVR-GCC		4.7.2
//				AVR-LIBC	1.8.0
//				binutils	2.20.1
//
//	2013.5.22	KAWAKAMI Yukio
//*******************************************************

#include	<avr/io.h>
#include	<avr/interrupt.h>
#include	<avr/eeprom.h>

#include	"main.h"

#define		LED_CURRENT		15		// LED電流(mA)
#define		BATT_LOW		3400	// ローバッテリー警告電圧(mV)
#define		BATT_LOW_VAL	(BATT_LOW * 1023L * R12_VAL / (R10_VAL + R12_VAL) / 1100)
#define		BATT_SHUTDOWN	3100	// シャットダウン電圧(mV)
#define		BATT_SHUTDOWN_VAL	(BATT_SHUTDOWN * 1023L * R12_VAL / (R10_VAL + R12_VAL) / 1100)

volatile uchar	WTimer;
volatile uchar	Timer;
volatile uchar	SwCount;
volatile ushort	BattVolt;

volatile uchar	LedOnTime;

//*******************************************************
//	タイマー０比較一致A割り込み
//*******************************************************
ISR(TIM0_COMPA_vect){
	static uchar	led_count = 0;

	LED_OFF;

	if (WTimer)	--WTimer;
	if (Timer)	--Timer;

	if (IS_SW){
		if (SwCount < 255)	SwCount++;
	} else {
		SwCount = 0;
	}

	if (++led_count >= 100){
		led_count = 0;
		if (LedOnTime && (led_count < LedOnTime)){
			LED_ON;
		}
	}

	PWM_ON;

	ADMUX = (1<<REFS0)|ADC_BATT;
	ADCSRA = (1<<ADEN)|(1<<ADSC)|(1<<ADIF)|(1<<ADIE)|(1<<ADPS1)|(1<<ADPS0);
}


//*******************************************************
//	タイマー０比較一致B割り込み
//*******************************************************
ISR(TIM0_COMPB_vect){

	PWM_OFF;
}


//*******************************************************
//	A/Dコンバータ割り込み
//*******************************************************
ISR(ADC_vect) {

	if ((ADMUX & 3) == ADC_BATT){
		BattVolt = (ushort)ADCL;
		BattVolt |= (((ushort)ADCH & 3) << 8);
		ADMUX = (1<<REFS0)|ADC_LED;
		ADCSRA = (1<<ADEN)|(1<<ADSC)|(1<<ADIF)|(1<<ADIE)|(1<<ADPS1)|(1<<ADPS0);
	} else {
		ushort	volt;
		volt = (ushort)ADCL;
		volt |= (((ushort)ADCH & 3) << 8);

		// LED 定電流制御
		if (volt > 0){
			long	l;
			l = (LED_CURRENT * R5_VAL * 1023L / 1100) * OCR0A;
			l /= volt;
			if (l < OCR0A){
				OCR0B = (uchar)l;
			} else {
				// 100%点灯
				OCR0B = 0xFF;
			}
		} else {
			// 100%点灯
			OCR0B = 0xFF;
		}

		ADCSRA &= ~(1<<ADEN);	// A/D停止
	}
}


//*******************************************************
//	10ms単位時間待ち
//*******************************************************
void wait_timer(uchar tm){

	WTimer = tm;

	while(WTimer){
		WDR;
		SLEEP;
	}
}


//*******************************************************
//	A/Dコンバータ値取得
//*******************************************************
ushort adc_in(uchar	ch){
	ushort	val;

	ADMUX = (1<<REFS0)|(ch & 3);		// 1.1V基準、右詰め
	ADCSRB = 0;
	// 1.2MHz / 8 = 150kHz
	ADCSRA = (1<<ADEN)|(1<<ADSC)|(1<<ADIF)|(1<<ADPS1)|(1<<ADPS0);
	while(!(ADCSRA & (1<<ADIF)));
	val = ADCL;
	val |= ((ushort)ADCH) << 8;

	return val;
}


//*******************************************************
//	メインルーチン
//*******************************************************
int main (void) {

	cli();	// 割り込み禁止

	PORTB = (1<<SWITCH)|(1<<PWM);
	DDRB = (1<<PLOCK);

	// スリープ動作設定
	MCUCR = (1<<SE);	// スリープ許可, アイドルモード

	// タイマ0設定
	TCCR0A = (1<<WGM01);				// CTCモード
	TCCR0B = (1<<CS01)|(1<<CS00);	// プリスケーラ 1/64
	OCR0A = F_CPU/64/100 - 1;		// 100Hz
	OCR0B = OCR0A / 8;	// 
	TIMSK0 = (1<<OCIE0A);	// 比較A割り込み許可
	TIMSK0 |= (1<<OCIE0B);	// 比較B割り込み許可

	ACSR |= (1<<ACD);	// アナログコンパレータ禁止

	// A/Dコンバータ デジタル入力OFF
	DIDR0 = (1<<ADC1D)|(1<<ADC2D);

	ADCSRB = 0;
	
	sei();

	wait_timer(20);		// 200ms

	POWER_LOCK;
	DDRB |= (1<<PWM);	// 点灯開始

	// メインループ
	for (;;){

		WDR;
		SLEEP;

		if (SwCount == 10){
			POWER_OFF;
		}

		if (BattVolt < BATT_SHUTDOWN_VAL){	// 3.1V以下でシャットダウン
			POWER_OFF;
		} else if (BattVolt < BATT_LOW_VAL){	// 3.3V以下で警告
			LedOnTime = 25;		// 250ms ON / 750ms OFF
		} else {
			LedOnTime = 0;
		}
	}
}
